/*
 * Copyright (c) 2006-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: XBayaDialog.java,v 1.9 2007/02/20 22:53:21 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.gui;

import java.awt.Container;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.Window;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;

import edu.indiana.extreme.xbaya.XBayaEngine;
import edu.indiana.extreme.xbaya.XBayaRuntimeException;
import edu.indiana.extreme.xbaya.util.SwingUtil;

/**
 * @author Satoshi Shirasuna
 */
public class XBayaDialog {

    private Window owner;

    private String title;

    private String description;

    private JDialog dialog;

    private JComponent mainPanel;

    private JComponent buttonPanel;

    /**
     * Constructs an XBayaDialog.
     * 
     * @param engine
     * @param title
     * @param mainPanel
     * @param buttonPanel
     */
    public XBayaDialog(XBayaEngine engine, String title,
            XBayaComponent mainPanel, XBayaComponent buttonPanel) {
        this(engine.getGUI().getFrame(), title, mainPanel.getSwingComponent(),
                buttonPanel.getSwingComponent());
    }

    /**
     * Constructs an XBayaDialog.
     * 
     * @param engine
     * @param title
     * @param mainPanel
     * @param buttonPanel
     */
    public XBayaDialog(XBayaEngine engine, String title,
            XBayaComponent mainPanel, JComponent buttonPanel) {
        this(engine.getGUI().getFrame(), title, mainPanel.getSwingComponent(),
                buttonPanel);
    }

    /**
     * Constructs an XBayaDialog.
     * 
     * @param engine
     * @param title
     * @param description
     * @param mainPanel
     * @param buttonPanel
     */
    public XBayaDialog(XBayaEngine engine, String title, String description,
            XBayaComponent mainPanel, JComponent buttonPanel) {
        this(engine.getGUI().getFrame(), title, description, mainPanel
                .getSwingComponent(), buttonPanel);
    }

    /**
     * Constructs an XBayaDialog.
     * 
     * @param engine
     * @param title
     * @param mainPanel
     * @param buttonPanel
     */
    public XBayaDialog(XBayaEngine engine, String title, JComponent mainPanel,
            JComponent buttonPanel) {
        this(engine.getGUI().getFrame(), title, mainPanel, buttonPanel);
    }

    /**
     * Constructs an XBayaDialog.
     * 
     * @param owner
     * @param title
     * @param mainPanel
     * @param buttonPanel
     */
    public XBayaDialog(Window owner, String title, XBayaComponent mainPanel,
            JComponent buttonPanel) {
        this(owner, title, mainPanel.getSwingComponent(), buttonPanel);
    }

    /**
     * Constructs an XBayaDialog.
     * 
     * @param owner
     * @param title
     * @param mainPanel
     * @param buttonPanel
     */
    public XBayaDialog(Window owner, String title, JComponent mainPanel,
            JComponent buttonPanel) {
        this(owner, title, null, mainPanel, buttonPanel);
    }

    /**
     * Constructs an XBayaDialog.
     * 
     * @param owner
     * @param title
     * @param description
     * @param mainPanel
     * @param buttonPanel
     */
    public XBayaDialog(Window owner, String title, String description,
            JComponent mainPanel, JComponent buttonPanel) {
        this.owner = owner;
        this.title = title;
        this.description = description;
        this.mainPanel = mainPanel;
        this.buttonPanel = buttonPanel;
        init();
    }

    /**
     * @return The dialog.
     */
    public JDialog getDialog() {
        return this.dialog;
    }

    /**
     * Determines whether this component should be visible when its parent is
     * visible. Components are initially visible, with the exception of top
     * level components such as <code>Frame</code> objects.
     * 
     * @return <code>true</code> if the component is visible,
     *         <code>false</code> otherwise
     */
    public boolean isVisible() {
        return this.dialog.isVisible();
    }

    /**
     * Shows the dialog.
     */
    public void show() {
        this.dialog.pack();

        // Adjust the size if it's bigger than the screen.
        Dimension size = this.dialog.getSize();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        final int inset = 100;
        int width = size.width;
        if (width > screenSize.width) {
            width = screenSize.width - inset;
        }
        int height = size.height;
        if (height > screenSize.height) {
            height = screenSize.height - inset;
        }
        this.dialog.setSize(width, height);

        this.dialog.setLocationRelativeTo(this.owner);
        this.dialog.setVisible(true);
    }

    /**
     * Hides the dialog.
     */
    public void hide() {
        this.dialog.setVisible(false);
    }

    /**
     * @param title
     */
    public void setTitle(String title) {
        this.title = title;
        this.dialog.setTitle(title);
    }

    /**
     * @param button
     */
    public void setDefaultButton(JButton button) {
        this.dialog.getRootPane().setDefaultButton(button);
    }

    private void init() {
        if (this.owner instanceof Frame) {
            this.dialog = new JDialog((Frame) this.owner);
        } else if (this.owner instanceof Dialog) {
            this.dialog = new JDialog((Dialog) this.owner);
        } else {
            // This should not happen.
            throw new XBayaRuntimeException(
                    "The owner component was neither Frame or Dialog.");
        }
        this.dialog.setTitle(this.title);
        this.dialog.setModal(true);
        this.dialog.setResizable(true);

        Container contentPane = this.dialog.getContentPane();
        int numRow = 0;
        if (this.description != null && this.description.length() > 0) {
            // TODO not working!!
            JLabel descriptionLabel = new JLabel("<html>" + this.description
                    + "</html>");

            contentPane.add(descriptionLabel);
            numRow++;
        }
        contentPane.add(this.mainPanel);
        numRow++;
        contentPane.add(this.buttonPanel);
        numRow++;
        SwingUtil.layoutToGrid(contentPane, numRow, 1, numRow - 2, 0);
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2006-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
